package evaluation;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.sf.jsqlparser.parser.CCJSqlParserManager;
import org.apache.derby.iapi.error.StandardException;
import org.apache.derby.impl.sql.compile.StatementNode;
import com.google.gson.Gson;

import evaluation.QueryStatusData.QueryStatus;

import testDataGen.GenerateCVC1;
import testDataGen.PopulateTestDataGrading;
import testDataGen.preProcessForDataGeneration;
import util.DatabaseConnection;
import util.DatabaseHelper;
import util.MyConnection;
import util.TableMap;

/*
 * Code to be used for hasing a table in postgres : 
 * SELECT        
    md5(CAST((array_agg(f.* order by id))AS text)) /* id is a primary key of table (to avoid random sorting) FROM  foo f; 
 */
/***
 * 
 * @author shree
 *
 */
public class TestAssignment {

	private static Logger logger = Logger.getLogger(TestAssignment.class.getName());

	/**
	 * This method is used for Automated Testing of Instructor and student
	 * queries
	 * 
	 * @param assignment_id
	 * @return
	 * @throws Exception
	 */
	public List<String> evaluateAssignment(int assignment_id) throws Exception {
		String done = "SELECT assignment_id from xdata_assignment where assignment_id=?";
		String sep = "|";
		try {
			Class.forName("org.postgresql.Driver");
		} catch (ClassNotFoundException e1) {
			logger.log(Level.SEVERE, e1.getMessage(), e1);
			// TODO Auto-generated catch block
			// e1.printStackTrace();
		}
		logger.log(Level.INFO, "Testing started");
		try (Connection dbcon = MyConnection.getDatabaseConnection()) {
			if (dbcon != null) {
				logger.log(Level.INFO, "Connected successfullly");
			}
			try (PreparedStatement donestmt = dbcon.prepareStatement(done)) {
				donestmt.setInt(1, assignment_id);
				try (ResultSet rs = donestmt.executeQuery()) {
					if (!rs.next()) {
						logger.log(Level.INFO, "Assignment id not valid");
						return null;
					}
					// rs.next();
					List<String> corr_queries = new ArrayList<String>();
					List<Integer> corr_query_id = new ArrayList<Integer>();
					String updateString = "update xdata_student_queries set verifiedcorrect=? where assignment_id=? and question_id=? and rollnum=?";
					String QueryString = "select query_id,correctquery from xdata_qinfo where assignment_id =?";
					TestAnswer test = new TestAnswer();

					try (PreparedStatement qstmt = dbcon.prepareStatement(QueryString)) {
						qstmt.setInt(1, assignment_id);
						try (ResultSet Queries = qstmt.executeQuery()) {

							while (Queries.next()) {
								corr_queries.add(Queries.getString(2));
								corr_query_id.add(Queries.getInt(1));
							}
						} // try block for resultset ends
					} // try block for qstmt ends
					List<String> corrAns = new ArrayList<String>();
					for (int i = 0; i < corr_queries.size(); i++) {
						String filePath = Integer.toString(assignment_id);
						preProcessForDataGeneration preProcess = new preProcessForDataGeneration();
						preProcess.generateDatasetForQuery(assignment_id, corr_query_id.get(i), 1, "AutomatedTesting", 0);
						//GenerateDataset_new g = new GenerateDataset_new(filePath);
						//g.generateDatasetForQuery(assignment_id, corr_query_id.get(i), 1, "AutomatedTesting", "true",
						//		corr_queries.get(i), "");
						String StudQueryString = "select rollnum,querystring from xdata_student_queries where question_id=? and assignment_id=?";
						try (PreparedStatement studQueriesStmt = dbcon.prepareStatement(StudQueryString)) {
							studQueriesStmt.setInt(1, corr_query_id.get(i));
							studQueriesStmt.setInt(2, assignment_id);
							try (ResultSet StudQueries = studQueriesStmt.executeQuery()) {
								while (StudQueries.next()) {
									int flag = 0;
									PreparedStatement upstmt = dbcon.prepareStatement(updateString);
									// SQLParser sqlParser = new SQLParser();
									// StatementNode s = null;
									String qry = StudQueries.getString("querystring");
									String rolln = StudQueries.getString("rollnum");
									String OriginalQry = qry.replaceAll("''", "'");
									logger.log(Level.INFO, "queryString" + OriginalQry);
									qry = OriginalQry.trim().replaceAll("\n+", " ");
									qry = qry.trim().replaceAll(" +", " ");
									logger.log(Level.INFO, "Cleansed Query is " + qry);

									if (flag == 0) {
										// FailedDataSetValues
										// fdValue=test.testAnswer(assignment_id,corr_query_id.get(i),"AutomatedTesting",
										// OriginalQry ,"student",filePath);
										FailedDataSetValues fdValue = test.testAnswer(assignment_id,
												corr_query_id.get(i), "AutomatedTesting", OriginalQry, rolln, filePath,
												false,null);
										logger.log(Level.INFO, "ans:" + fdValue.getStatus());
										if (fdValue.getStatus().equalsIgnoreCase("failed")) {
											upstmt.setBoolean(1, false);
											corrAns.add("FAIL" + sep + assignment_id + sep + corr_query_id.get(i) + sep
													+ StudQueries.getString(2));
										} else {
											upstmt.setBoolean(1, true);
											corrAns.add("PASS" + sep + assignment_id + sep + corr_query_id.get(i) + sep
													+ StudQueries.getString(2));
										}
									}
									upstmt.setInt(2, assignment_id);
									upstmt.setInt(3, corr_query_id.get(i));
									upstmt.setString(4, StudQueries.getString(1));
									upstmt.executeUpdate();

								}
							} // try block for StudQueries
						} // try block for studQueriesStmt ends

					}
					for (int ind = 0; ind < corrAns.size(); ind++) {
						logger.log(Level.INFO, "Result:" + corrAns.get(ind));
					}
				
					return corrAns;
				} // try block for result set ends
			} // try block to close donestmt ends
		} catch (SQLException ex) {
			System.err.println("SQLEEException: " + ex.getMessage());
			logger.log(Level.SEVERE, ex.getMessage(), ex);
			// ex.printStackTrace();
		}
		return null;
	}

	/**
	 * This method is invoked during the late submission of answers Triggered by
	 * instructor.
	 * 
	 */
	public boolean evaluateLateSubmission(Connection dbcon, Connection testCon, String[] args) throws SQLException {	//Nd-106
		boolean status = false;

		int assignment_id = Integer.parseInt(args[0]);

		String rollNum = args[1];
		String courseId = args[2];
		int noOfQuestions = Integer.parseInt(args[3]);
		int marksToBeReduced = Integer.parseInt(args[4]);
		PopulateTestDataGrading populateTestData = new PopulateTestDataGrading();
		TestAnswer test = new TestAnswer();
		// int question_id = 1;
		try {	//Nd-115
	/*segment116-269:begins*/	
			for (int ij = 1; ij < noOfQuestions; ij++) {
				QueryStatus queryStatus = QueryStatus.Correct;
				boolean queryExists = false;
				
				populateTestData.deleteAllTempTablesFromTestUser(testCon);
				GenerateCVC1 cvc = new GenerateCVC1();											
				preProcessForDataGeneration preProcess = new preProcessForDataGeneration();
				
				cvc.setAssignmentId(assignment_id);
				cvc.setQuestionId(ij);
				cvc.setQueryId(1);
				cvc.setCourseId(courseId);
				preProcess.initializeConnectionDetails(cvc);
			 	
				TableMap tm = cvc.getTableMap();

				String json1 = "";
				int question_id = ij;
				String updateString = "update xdata_student_queries set tajudgement=?, result = ? "
						+ "where assignment_id=? and question_id = ? and rollnum=? and course_id=?";

				PreparedStatement upstmt = dbcon.prepareStatement(updateString);

				// Get student answer for each question
				String getAnswersString = "select * from xdata_student_queries where assignment_id=? and rollnum=? and course_id=? and question_id=?";
				try (PreparedStatement getstmt = dbcon.prepareStatement(getAnswersString)) {
					getstmt.setInt(1, assignment_id);
					getstmt.setString(2, rollNum);
					getstmt.setString(3, courseId);
					getstmt.setInt(4, ij);

					try (ResultSet rs = getstmt.executeQuery()) {
						while (rs.next()) {	//Nd-143
							int flag = 0;
							HashSet<String> hs = new HashSet<String>();	//belongs to segment 150-246
							Gson gson = new Gson();//belongs to segment 150-246
							String json = "";
							boolean isLateSubmission = rs.getBoolean("late_submission_flag");//belongs to segment 150-246
							// Start of student Query evaluation for a question
							// Do run student query against the default dataset
							// and datasets generated by XData
							String testQuery = rs.getString("querystring");
				/*segment145,146,148,150-246: begins*/
							if (testQuery != null && !testQuery.isEmpty()) {	//Nd-151
								queryExists = true;
						/*segment153-168: begins*/		
								try {
									testQuery = testQuery.trim().replaceAll("\n+", " ");
									testQuery = testQuery.trim().replaceAll(" +", " ");

									populateTestData.deleteAllTempTablesFromTestUser(testCon);
									populateTestData.createTempTables(testCon, assignment_id, question_id);
									try (PreparedStatement testStatement = testCon.prepareStatement(testQuery)) {
										testStatement.executeQuery();
									}
								} catch (Exception e) {
									logger.log(Level.SEVERE, e.getMessage(), e);
									upstmt.setBoolean(1, true);
									flag = 1;
								}
					/*segment153-168: ends*/	
								if (flag == 0) {	//Nd-169
									String ans = "";
									try {
										String queryId = "A" + assignment_id + "Q" + question_id + "S" + 1;
										// FailedDataSetValues failedDs =
										// test.testAnswerMatchAllOption(assignment_id,question_id,courseId,
										// OriginalQry, rollNum,
										// "4/"+courseId+"/"+queryId);
										FailedDataSetValues failedDs = test.testAnswer(assignment_id, question_id,
												courseId, testQuery, rollNum, "4/" + courseId + "/" + queryId,
												isLateSubmission,null);
					/*segment174-225: begins*/
										if (failedDs.getStatus().equalsIgnoreCase("Failed")) {//Nd-176
											Map<String, Map<String, ArrayList<String>>> failedStudDataMap = new HashMap<String, Map<String, ArrayList<String>>>();	//belongs to segment below
											Map<String, Map<String, ArrayList<String>>> failedInstrDataMap = new HashMap<String, Map<String, ArrayList<String>>>();//belongs to segment below

											hs.add(failedDs.getDataSetIdList().get(0));
										
						/*segment177,178,182-220: begins*/				
											String add_datasets = "insert into xdata_detectdataset values (?,?,?,?,?,?,?)";
											try (PreparedStatement add = dbcon.prepareStatement(add_datasets)) {

												for (int i = 0; i < failedDs.getDataSetIdList().size(); i++) {

													String dataset = failedDs.getDataSetIdList().get(i);
													// Load the failed dataset
													// into temp tables
													ArrayList<FailedColumnValues> failedList = new ArrayList<FailedColumnValues>();
													Map<String, ArrayList<String>> failedColMap = new HashMap<String, ArrayList<String>>();
													logger.log(Level.INFO, "Inside dataset list");

													// p.fetchAndPopulateTestDatabase(dbcon,
													// testCon,
													// assignment_id,question_id,1,
													// course_Id, dataset, tm);
													if (dataset.startsWith("DS")) {
														if (dataset.equalsIgnoreCase("DS_Default")) {
															populateTestData.deleteAllTempTablesFromTestUser(testCon);
															populateTestData.createTempTableData(dbcon, testCon,
																	assignment_id, question_id, courseId);
														} else {	//Nd-200
															populateTestData.fetchAndPopulateTestDatabase(dbcon,
																	testCon, assignment_id, question_id, 1, courseId,
																	dataset, tm);
														}
													} else {//Nd-202
														populateTestData.createTempTableWithDefaultData(dbcon, testCon,
																assignment_id, question_id, courseId, dataset);
													}
													failedDs = this.getStudentOutput(testCon, dataset, testQuery,
															failedList, failedDs, failedStudDataMap, false);
													failedDs = this.getInstructorOutput(testCon, dataset,
															failedDs.getInstrQuery(), failedList, failedDs,
															failedInstrDataMap, false);

													logger.log(Level.INFO, "Answer is :" + ans);
													failedDs.getDsValueMap().put(dataset, failedList);
													json = gson.toJson(failedDs);
													logger.log(Level.INFO, "JSON VALUE OF FAILED DATASETS :" + json);
													add.setString(1, rollNum);
													add.setString(2, queryId);
													add.setString(3, dataset);
													add.setString(4, json);
													add.setInt(5, assignment_id);
													add.setInt(6, question_id);
													add.setString(7, courseId);
													// add.setString(4, );
													add.executeUpdate();

												}
											}
							/*segment182-220: ends*/				
											json1 = gson.toJson(failedDs);
											upstmt.setBoolean(1, true);
											
										} else {	//Nd-223
											queryStatus = QueryStatus.Correct;
											upstmt.setBoolean(1, true);

										}
					/*segment174-225: ends*/					
										upstmt.setString(2, json);
										cvc.closeConn();
									} catch (Exception e) {	//Nd-228
										logger.log(Level.SEVERE, "Exception caught here:", e);
										// e.printStackTrace();
										queryStatus = QueryStatus.Error;
										upstmt.setBoolean(1, true);

									}

								}
								upstmt.setString(2, json);

								upstmt.setInt(3, assignment_id);
								upstmt.setInt(4, question_id);
								// upstmt.setInt(3, question_id);
								upstmt.setString(5, testQuery);
								upstmt.setString(6, courseId);
								upstmt.executeUpdate();
							} else {	//Nd-239

								upstmt.setBoolean(1, false);
								upstmt.setString(2, null);
								upstmt.setInt(3, assignment_id);
								upstmt.setInt(4, question_id);
								upstmt.setString(5, testQuery);
								upstmt.setString(6, courseId);

								upstmt.executeUpdate();
							}

							// If student answer exists

							// End of student Query evaluation for a question
						}
					}
				}
				if (!queryExists) {	//Nd-247
					String qryUpdate = "update xdata_student_queries set verifiedcorrect = false where assignment_id = '"
							+ assignment_id + "' and question_id='" + question_id + "' and rollnum = '" + rollNum
							+ "' and course_id= '" + courseId + "'";
					try (PreparedStatement pstmt3 = dbcon.prepareStatement(qryUpdate)) {
						pstmt3.executeUpdate();
					}
					queryStatus = QueryStatus.NoDataset;
				} else if (queryStatus == QueryStatus.Correct) {	//Nd-253
					int marksAfterReduction = 100 - marksToBeReduced;
					String qryUpdate = "update xdata_student_queries set verifiedcorrect = true,score="
							+ marksAfterReduction + " where assignment_id = '" + assignment_id + "' and question_id='"
							+ question_id + "' and rollnum = '" + rollNum + "'and course_id= '" + courseId + "'";
					try (PreparedStatement pstmt3 = dbcon.prepareStatement(qryUpdate)) {
						pstmt3.executeUpdate();
					}
				} else {	//Nd-260
					String qryUpdate = "update xdata_student_queries set verifiedcorrect = false, result=? where assignment_id = '"
							+ assignment_id + "' and question_id='" + question_id + "' and rollnum = '" + rollNum
							+ "'and course_id= '" + courseId + "'";
					try (PreparedStatement pstmt3 = dbcon.prepareStatement(qryUpdate)) {
						pstmt3.setString(1, json1);
						pstmt3.executeUpdate();
					}
				}
				if (ij == noOfQuestions) {	//Nd-266
					status = true;
					break;
				}
			} // End of for loop for number of questions
	/*segment116-269	*/	
		} catch (Exception e) {
			logger.log(Level.SEVERE, "Exception caught at end of TestAssignment.evaluate function :", e);
		}

		return status;
	}

	/**
	 * This method tests the passed student query across the instructor answer and with out saving anything in database. 
	 * 
	 * @param dbcon
	 * @param testCon
	 * @param args
	 * @return
	 * @throws SQLException
	 */
	public FailedDataSetValues evaluateGuestAnswer(Connection dbcon, Connection testCon, String[] args) throws SQLException {	//Nd-274
		TestAnswer test = new TestAnswer();
		QueryStatus queryStatus = QueryStatus.Correct;
		boolean queryExists = false;
		QueryStatusData statusData = new QueryStatusData();
		PopulateTestDataGrading populateTestData = new PopulateTestDataGrading();

		int assignment_id = Integer.parseInt(args[0]);
		int question_id = Integer.parseInt(args[1]);

		String qId = "A" + assignment_id + "Q" + question_id + "S" + 1;
		String rollNum = args[2];
		String courseId = args[3];
		String StudQueryString = args[4];
		String studRole = args[5];
		String json = "";
		int flag = 0;
		FailedDataSetValues failedDs = new FailedDataSetValues();	//belongs to segment 289-312
		Gson gson = new Gson();
	
/*segment289-312: begins*/		
		String testQuery = StudQueryString;
		
		try {			//Nd-292
			logger.log(Level.INFO, "queryString" + testQuery);
			testQuery = testQuery.trim().replaceAll("\n+", " ");
			testQuery = testQuery.trim().replaceAll(" +", " ");

			
			populateTestData.deleteAllTempTablesFromTestUser(testCon);
			populateTestData.createTempTables(testCon, assignment_id, question_id);
			try (PreparedStatement testStatement = testCon.prepareStatement(testQuery)) {	//Nd-300
				testStatement.executeQuery();
			}
		} catch (Exception e) {	//Nd-303
			logger.log(Level.SEVERE, e.getMessage(), e);
			// e.printStackTrace();
			statusData.ErrorMessage = e.getMessage();
			queryStatus = QueryStatus.Error;	
			failedDs.setStatus("Error");
			failedDs.setErrorMessage(e.getMessage());
			flag = 1;
		}
/*Segment289-312: ends*/		
		if (flag == 0) {	//Nd-313
			String ans = "";
			HashSet<String> hs = new HashSet<String>();
			try {
				String queryId = "A" + assignment_id + "Q" + question_id + "S" + 1;
				failedDs = test.testAnswer(assignment_id, question_id, courseId,
						StudQueryString, rollNum, "4/" + courseId + "/" + queryId, false,studRole);
				GenerateCVC1 cvc = new GenerateCVC1();											
				preProcessForDataGeneration preProcess = new preProcessForDataGeneration();
				
				cvc.setAssignmentId(assignment_id);
				cvc.setQuestionId(question_id);
				cvc.setQueryId(1);
				cvc.setCourseId(courseId);
				preProcess.initializeConnectionDetails(cvc);
			 	
				TableMap tm = cvc.getTableMap();

				if (failedDs.getStatus().equalsIgnoreCase("Failed")) {	//Nd-329
					
					// Get DS0	//below two stmts failedStudDataMap and faileInstrDataMap belongs to segment 330-363
					Map<String, Map<String, ArrayList<String>>> failedStudDataMap = new HashMap<String, Map<String, ArrayList<String>>>();
					Map<String, Map<String, ArrayList<String>>> failedInstrDataMap = new HashMap<String, Map<String, ArrayList<String>>>();

					hs.add(failedDs.getDataSetIdList().get(0));
				
			/*segment330-363: begin*/
					for (int i = 0; i < failedDs.getDataSetIdList().size(); i++) {	//Nd-338

						String dataset = failedDs.getDataSetIdList().get(i);
						// Load the failed dataset into
						// temp tables
						ArrayList<FailedColumnValues> failedList = new ArrayList<FailedColumnValues>();
						Map<String, ArrayList<String>> failedColMap = new HashMap<String, ArrayList<String>>();
						logger.log(Level.INFO, "Inside dataset list");

						if (dataset.startsWith("DS")) {	//Nd-345
							if (dataset.equalsIgnoreCase("DS_Default")) {
								populateTestData.deleteAllTempTablesFromTestUser(testCon);
								populateTestData.createTempTableData(dbcon, testCon,
										assignment_id, question_id, courseId);
							} else {	//Nd-350
								populateTestData.fetchAndPopulateTestDatabase(dbcon, testCon,
										assignment_id, question_id, 1, courseId, dataset, tm);
							}

						} else {	//Nd-352
							populateTestData.createTempTableWithDefaultData(dbcon, testCon,
									assignment_id, question_id, courseId, dataset);
							
						}
						failedDs = this.getStudentOutput(testCon, dataset, StudQueryString,
								failedList, failedDs, failedStudDataMap, false);
						failedDs = this.getInstructorOutput(testCon, dataset,
								failedDs.getInstrQuery(), failedList, failedDs,
								failedInstrDataMap, false);

						logger.log(Level.INFO, "Answer is :" + ans);
						failedDs.getDsValueMap().put(dataset, failedList);
						json = gson.toJson(failedDs);
						logger.log(Level.INFO, "JSON VALUE OF FAILED DATASETS :" + json);

						queryStatus = QueryStatus.Incorrect;
					}
		/*segment330-363: ends*/			
				}
			}catch (Exception e) {	//Nd-364
				logger.log(Level.SEVERE, e.getMessage(), e);
				//statusData.ErrorMessage = e.getMessage();
				failedDs.setErrorMessage(e.getMessage());
				failedDs.setStatus(QueryStatus.Error.toString());			
				flag = 1;
			}
			
		}if (StudQueryString != null && StudQueryString.equals("")) {	//Nd-374
			failedDs.setStatus(QueryStatus.NoDataset.toString());
		}
		return failedDs;
		
	}
	
	/**
	 * This method is called for evaluating student query in interactive mode
	 * 
	 * @param dbcon
	 * @param testCon
	 * @param args
	 * @return
	 * @throws SQLException
	 * @throws StandardException
	 */
	public QueryStatusData evaluateQuestion(Connection dbcon, Connection testCon, String[] args) throws SQLException {	//Nd-378
		String StudQueryString = "select querystring from xdata_student_queries where " + "assignment_id =?  and "
				+ "question_id = ? and rollnum=?";
		TestAnswer test = new TestAnswer();
		QueryStatus queryStatus = QueryStatus.Correct;
		boolean queryExists = false;
		QueryStatusData statusData = new QueryStatusData();
		PopulateTestDataGrading populateTestData = new PopulateTestDataGrading();

		int assignment_id = Integer.parseInt(args[0]);
		int question_id = Integer.parseInt(args[1]);

		String qId = "A" + assignment_id + "Q" + question_id + "S" + 1;
		String rollNum = args[2];
		String courseId = args[3];
		String json1 = "";
		// try(PreparedStatement studQueriesStmt =
		// dbcon.prepareStatement(StudQueryString)){
		String updateString = "update xdata_student_queries set tajudgement=?, result = ? "
				+ "where assignment_id=? and question_id = ? and rollnum=? and course_id=?";

		try (PreparedStatement upstmt = dbcon.prepareStatement(updateString)) {
			try (PreparedStatement studQueriesStmt = dbcon.prepareStatement(StudQueryString)) {

				studQueriesStmt.setInt(1, assignment_id);
				studQueriesStmt.setInt(2, question_id);
				studQueriesStmt.setString(3, rollNum);
				logger.log(Level.INFO, studQueriesStmt.toString());
				try (ResultSet StudQueries = studQueriesStmt.executeQuery()) {
					HashSet<String> hs = new HashSet<String>();
					if (StudQueries.next()) {	//Nd-405
					/*segment 406-534: begins*/
						Gson gson = new Gson();
						String json = "";
						int flag = 0;
						hs.clear();
						// SQLParser sqlParser = new SQLParser();
						CCJSqlParserManager pm = new CCJSqlParserManager();
						StatementNode s = null;
						String qry = StudQueries.getString("querystring");
						String testQuery = StudQueries.getString("querystring");
						
						if (qry != null && !qry.isEmpty() && testQuery != null && !testQuery.isEmpty()) {//Nd-416
							queryExists = true;
							String OriginalQry = qry.replaceAll("''", "'");
							logger.log(Level.INFO, "queryString" + OriginalQry);
							qry = OriginalQry.trim().replaceAll("\n+", " ");
							qry = qry.trim().replaceAll(" +", " ");
							logger.log(Level.INFO, "Cleansed Query is " + qry);
							qry = qry.replace("NATURAL LEFT OUTER", "NATURAL");
							qry = qry.replace("NATURAL RIGHT OUTER", "NATURAL");
				/*segment427-445:begins*/		
							try {
								logger.log(Level.INFO, "queryString" + testQuery);
								testQuery = testQuery.trim().replaceAll("\n+", " ");
								testQuery = testQuery.trim().replaceAll(" +", " ");

								
								populateTestData.deleteAllTempTablesFromTestUser(testCon);
								populateTestData.createTempTables(testCon, assignment_id, question_id);
								try (PreparedStatement testStatement = testCon.prepareStatement(testQuery)) {
									testStatement.executeQuery();
								}
							} catch (Exception e) {
								logger.log(Level.SEVERE, e.getMessage(), e);
								// e.printStackTrace();
								statusData.ErrorMessage = e.getMessage();
								queryStatus = QueryStatus.Error;
								upstmt.setBoolean(1, true);
								flag = 1;
							}
				/*segment427-445: ends*/			
							if (flag == 0) {	//Nd-446
								String ans = "";
								try {
									String queryId = "A" + assignment_id + "Q" + question_id + "S" + 1;
									FailedDataSetValues failedDs = test.testAnswer(assignment_id, question_id, courseId,
											OriginalQry, rollNum, "4/" + courseId + "/" + queryId, false,null);
									GenerateCVC1 cvc = new GenerateCVC1();											
									preProcessForDataGeneration preProcess = new preProcessForDataGeneration();
									
									cvc.setAssignmentId(assignment_id);
									cvc.setQuestionId(question_id);
									cvc.setQueryId(1);
									cvc.setCourseId(courseId);
									preProcess.initializeConnectionDetails(cvc);
					/*segment458-511: begins*/			 	
									TableMap tm = cvc.getTableMap();
									if (failedDs.getStatus().equalsIgnoreCase("Failed")) {//Nd-461
										
										// Get DS0
										Map<String, Map<String, ArrayList<String>>> failedStudDataMap = new HashMap<String, Map<String, ArrayList<String>>>();	//belongs to 462-506
										Map<String, Map<String, ArrayList<String>>> failedInstrDataMap = new HashMap<String, Map<String, ArrayList<String>>>();//belongs to 462-506

										hs.add(failedDs.getDataSetIdList().get(0));	//Does not belong to this segment
										
						/*segment462-506: begins*/ //Include 2 stmt above marked				
										
										String add_datasets = "insert into xdata_detectdataset values (?,?,?,?,?,?,?)";
										try (PreparedStatement add = dbcon.prepareStatement(add_datasets)) {

											for (int i = 0; i < failedDs.getDataSetIdList().size(); i++) {

												String dataset = failedDs.getDataSetIdList().get(i);
												// Load the failed dataset into
												// temp tables
												ArrayList<FailedColumnValues> failedList = new ArrayList<FailedColumnValues>();
												Map<String, ArrayList<String>> failedColMap = new HashMap<String, ArrayList<String>>();
												logger.log(Level.INFO, "Inside dataset list");

												if (dataset.startsWith("DS")) {
													if (dataset.equalsIgnoreCase("DS_Default")) {
														populateTestData.deleteAllTempTablesFromTestUser(testCon);
														populateTestData.createTempTableData(dbcon, testCon,
																assignment_id, question_id, courseId);
													} else {
														populateTestData.fetchAndPopulateTestDatabase(dbcon, testCon,
																assignment_id, question_id, 1, courseId, dataset, tm);
													}

												} else {
													populateTestData.createTempTableWithDefaultData(dbcon, testCon,
															assignment_id, question_id, courseId, dataset);
													
												}
												failedDs = this.getStudentOutput(testCon, dataset, OriginalQry,
														failedList, failedDs, failedStudDataMap, false);
												failedDs = this.getInstructorOutput(testCon, dataset,
														failedDs.getInstrQuery(), failedList, failedDs,
														failedInstrDataMap, false);

												logger.log(Level.INFO, "Answer is :" + ans);
												failedDs.getDsValueMap().put(dataset, failedList);
												json = gson.toJson(failedDs);
												logger.log(Level.INFO, "JSON VALUE OF FAILED DATASETS :" + json);

												add.setString(1, rollNum);
												add.setString(2, queryId);
												add.setString(3, dataset);
												add.setString(4, json);
												add.setInt(5, assignment_id);
												add.setInt(6, question_id);
												add.setString(7, courseId);
												// add.setString(4, );
												add.executeUpdate();
												queryStatus = QueryStatus.Incorrect;
											}

										}
										
					/*segment462-506: ends*/					
										json1 = gson.toJson(failedDs);
										upstmt.setBoolean(1, true);
									} else {	//Nd-509
										queryStatus = QueryStatus.Correct;
										upstmt.setBoolean(1, true);

									}
				/*segment458-511: ends*/					
									upstmt.setString(2, json);
									cvc.closeConn();
								} catch (Exception e) {
									logger.log(Level.SEVERE, "Exception caught here:", e);
									// e.printStackTrace();
									queryStatus = QueryStatus.Error;
									upstmt.setBoolean(1, true);
								}

							}
							upstmt.setString(2, json);

							upstmt.setInt(3, assignment_id);
							upstmt.setInt(4, question_id);
							// upstmt.setInt(3, question_id);
							upstmt.setString(5, StudQueries.getString(1));
							upstmt.setString(6, courseId);
							upstmt.executeUpdate();
						} else {	//Nd-526

							upstmt.setBoolean(1, false);
							upstmt.setString(2, null);
							upstmt.setInt(3, assignment_id);
							upstmt.setInt(4, question_id);
							upstmt.setString(5, StudQueries.getString(1));
							upstmt.setString(6, courseId);

							upstmt.executeUpdate();
						}
		/*Segment406-534 : ends*/	
					}
				
				} // close StudQueries resultset
			} // close Studqueries statement

		}
/*segment535-553: begins*/
		if (!queryExists) {
			String qryUpdate = "update xdata_student_queries set verifiedcorrect = false where assignment_id = '"
					+ assignment_id + "' and question_id='" + question_id + "' and rollnum = '" + rollNum
					+ "' and course_id= '" + courseId + "'";
			try (PreparedStatement pstmt3 = dbcon.prepareStatement(qryUpdate)) {
				pstmt3.executeUpdate();
			}
			queryStatus = QueryStatus.NoDataset;
		} else if (queryStatus == QueryStatus.Correct) {
			String qryUpdate = "update xdata_student_queries set verifiedcorrect = true,score = 100 where assignment_id = '"
					+ assignment_id + "' and question_id='" + question_id + "' and rollnum = '" + rollNum
					+ "' and course_id= '" + courseId + "'";
			try (PreparedStatement pstmt3 = dbcon.prepareStatement(qryUpdate)) {
				pstmt3.executeUpdate();
			}
		} else {
			String qryUpdate = "update xdata_student_queries set verifiedcorrect = false, result=? where assignment_id = '"
					+ assignment_id + "' and question_id='" + question_id + "' and rollnum = '" + rollNum
					+ "' and course_id= '" + courseId + "'";
			try (PreparedStatement pstmt3 = dbcon.prepareStatement(qryUpdate)) {
				pstmt3.setString(1, json1);
				pstmt3.executeUpdate();
			}
		}
		statusData.Status = queryStatus;
/*segment535-553 : ends*/		
		return statusData;
	}

	/**
	 * To be used by instructor when evaluate assignment or evaluate questions link is clicked from UI
	 * 
	 * This gets all student answers, get the correct instructor_queries and loop thru the instructor queries and 
	 * then for each instructor query, for each dataset, run all student queries and get the output data for failed students
	 * save the result in DB.
	 * 
	 * @return
	 */
	public void newEvaluateAssignment(Connection dbcon, String[] args) throws SQLException, Exception {	//Nd-555
		
		String StudQueryString = "select querystring,rollnum from xdata_student_queries where " + "assignment_id =?  and "
									+ "question_id = ?";
		int assignment_id = Integer.parseInt(args[0]);
		int question_id = Integer.parseInt(args[1]);
		String courseId = args[2];		
		ArrayList<String> studentRollNums = new ArrayList<String>();
		ArrayList<String> studentQueries = new ArrayList<String>();
		TestAnswer test = new TestAnswer();
		
		String updateString = "update xdata_student_queries set tajudgement=?, result = ? "
				+ "where assignment_id=? and question_id = ? and rollnum=? and course_id=?";

		try (PreparedStatement upstmt = dbcon.prepareStatement(updateString)) {
				try (PreparedStatement studQueriesStmt = dbcon.prepareStatement(StudQueryString)) {
	
						studQueriesStmt.setInt(1, assignment_id);
						studQueriesStmt.setInt(2, question_id);
						logger.log(Level.INFO, studQueriesStmt.toString());
						try (ResultSet StudQueries = studQueriesStmt.executeQuery()) {
							while(StudQueries.next()) {
								studentRollNums.add(StudQueries.getString("rollnum"));
								String studQuery = StudQueries.getString("querystring");
								if(studQuery != null && !studQuery.isEmpty() && studQuery != ""){	//Nd-580
									studentQueries.add(StudQueries.getString("querystring"));
								}else{	//Nd-583
									//If student has not answered the question, make it null for evaluation
									studentQueries.add(null);
								}
							}
						}//Got all student queries and rollnumbers				
						String queryId = "A" + assignment_id + "Q" + question_id + "S" + 1;
						ArrayList<FailedDataSetValues> failedList = test.newTestAnswer(assignment_id, question_id, courseId,
								studentQueries,studentRollNums,"4/" + courseId + "/" + queryId,"");				
						// Get failedDataSets list - iterate on the list, get stud id and update xdata_students_queries table with the json obj and marks
					 	 // update student table accordingly
						String updateStudentTableString = "update xdata_student_queries set tajudgement=?, result = ?  where assignment_id=? and question_id = ? and rollnum=? and course_id=?";
						Gson gson = new Gson();
						String json = "";
						int count = 1;
						try (PreparedStatement updtstmt = dbcon.prepareStatement(updateStudentTableString)) {	//Nd-591				
								for(FailedDataSetValues fdv :  failedList){							
										json = gson.toJson(fdv);
										upstmt.setBoolean(1, true);
										upstmt.setString(2, json);
										upstmt.setInt(3, assignment_id);
										upstmt.setInt(4, question_id);
										upstmt.setString(5,fdv.getStudentRollNo());
										upstmt.setString(6, courseId);
										upstmt.executeUpdate();
										logger.log(Level.WARNING,"***************************************************");
										logger.log(Level.WARNING,"FailedDs Item : Roll No :: " + count +" :::: "+fdv.getStudentRollNo());						
										logger.log(Level.WARNING,"FailedDs Status :: " + fdv.getStatus());
										logger.log(Level.WARNING,"***************************************************");
										count++;
								}		
						}catch (Exception e) {		//Nd-611
							logger.log(Level.SEVERE, "Exception caught here: " + e.getMessage(), e);
							upstmt.setBoolean(1, true);
						}	
			}
		}		
		logger.log(Level.INFO, "Evaluation of Question "+question_id+" in Assignment "+assignment_id+" completed. ");
	}
	/**
	 * This method return the <i>student query's</i> output for the loaded data set
	 * Used to get results of student query on the failed data set for
	 * displaying failed test cases
	 * 
	 * @param testCon
	 * @param dataSetId
	 * @param OriginalQry
	 * @param failedList
	 * @param failedDs
	 * @param failedStudDataMap
	 * @param isDefaultDS
	 * @return
	 * @throws SQLException
	 */
	public FailedDataSetValues getStudentOutput(Connection testCon, String dataSetId, String OriginalQry,
			ArrayList<FailedColumnValues> failedList, FailedDataSetValues failedDs,
			Map<String, Map<String, ArrayList<String>>> failedStudDataMap, boolean isDefaultDS) {	//Nd-617
		Map<String, ArrayList<String>> failedColMap = new HashMap<String, ArrayList<String>>();

		try {
			// execute student query on the failed DS and get the output
			try (PreparedStatement pp = testCon.prepareStatement(OriginalQry)) {
				try (ResultSet rr = pp.executeQuery()) {
					ResultSetMetaData metadata = rr.getMetaData();
					int no_of_columns = metadata.getColumnCount();
					String result = "";
					String columnName = "";
					List<String> existingColNames = new ArrayList<String>();  
					int index = 1;
					for(int cl=1;cl<=no_of_columns;cl++)
					{
						existingColNames.add(metadata.getColumnName(cl));
					}
					for (int cl = 1; cl <= no_of_columns; cl++) {
						ArrayList<String> values = new ArrayList<String>();
						FailedColumnValues failedColumns = new FailedColumnValues();
						columnName = metadata.getColumnName(cl);
						
						existingColNames.remove(metadata.getColumnName(cl));
			/*Segment642-651 : begins*/			
						//After removing the column name, if still colName exists it is duplicate column- so suffix with index.
						if(existingColNames.contains(metadata.getColumnName(cl))){//Nd-644
							
							columnName = metadata.getColumnName(cl)+index;
							index ++;
							existingColNames.add(metadata.getColumnName(cl));							
						}else{	//Nd-650
							columnName = metadata.getColumnName(cl);							
						}
		/*Segment642-651 : ends*/
						try (ResultSet rr1 = pp.executeQuery()) {	//Nd-652
							metadata = rr1.getMetaData();
							while (rr1.next()) {
								// while(rr.next())
								// {
								int type = metadata.getColumnType(cl);
								values.add(rr1.getString(cl));
							}
													
							failedColumns.setColumnName(metadata.getColumnName(cl));
							failedColumns.setValues(values);
							failedColMap.put(columnName, values);
							failedList.add(failedColumns);
						}
					}
				} // try block to close rr resultset
			} // try block for pp statement ends
		} catch (SQLException e) {	//Nd-666
			logger.log(Level.SEVERE,
					"Student test case output generation fails because of syntax error :" + e.getMessage(), e);
			
		}
		failedStudDataMap.put(dataSetId, failedColMap);
		failedDs.setStudentQueryOutput(failedStudDataMap);
		return failedDs;
	}

	/**
	 * This method return the <i>instructor query's</i> output for the loaded dataset
	 * Used to get results of instructor query on the failed dataset for
	 * displaying failed test cases
	 * 
	 * @param testCon
	 * @param dataSetId
	 * @param OriginalQry
	 * @param failedList
	 * @param failedDs
	 * @param failedStudDataMap
	 * @param isDefaultDS
	 * @return
	 * @throws SQLException
	 */
	public FailedDataSetValues getInstructorOutput(Connection testCon, String dataSetId, String OriginalQry,
			ArrayList<FailedColumnValues> failedList, FailedDataSetValues failedDs,
			Map<String, Map<String, ArrayList<String>>> failedInstrDataMap, boolean isDefaultDS) {	//Nd-673

		Map<String, ArrayList<String>> failedInstrColMap = new HashMap<String, ArrayList<String>>();
		try {
			try (PreparedStatement pp = testCon.prepareStatement(OriginalQry)) {
				try (ResultSet rr = pp.executeQuery()) {
					ResultSetMetaData metadata = rr.getMetaData();
					int no_of_columns = metadata.getColumnCount();
					String result = "";
					String columnName = "";
					// ArrayList <String> values = new ArrayList<String>();
					List<String> existingColNames = new ArrayList<String>();  
					int index = 1;
					for(int cl=1;cl<=no_of_columns;cl++)
					{
						existingColNames.add(metadata.getColumnName(cl));
					}
					
					for (int cl = 1; cl <= no_of_columns; cl++) {
						ArrayList<String> values = new ArrayList<String>();
						FailedColumnValues failedColumns = new FailedColumnValues();
						// out_assignment.println("<th>"+metadata.getColumnLabel(cl)+"</th>");
						// result+=metadata.getColumnName(cl)+"@@";
						// failedDs.getColValueMap().add(metadata.getColumnName(cl));
						columnName = metadata.getColumnName(cl);
						
						existingColNames.remove(metadata.getColumnName(cl));
				/*Segment698-707: begins*/		//contains 1 IR extra from above instruction	
						//After removing , if still coName exists it is duplicate column- so suffix with index.
						if(existingColNames.contains(metadata.getColumnName(cl))){	//Nd-700
							
							columnName = metadata.getColumnName(cl)+index;
							index ++;
							existingColNames.add(metadata.getColumnName(cl));
							
						}else{	//Nd-706
							columnName = metadata.getColumnName(cl);
							
						}
						
			/*segment698-707 : ends*/			
						try (ResultSet rr1 = pp.executeQuery()) {	//Nd-708
							metadata = rr1.getMetaData();
							while (rr1.next()) {
								int type = metadata.getColumnType(cl);
								values.add(rr1.getString(cl));

							}

							failedColumns.setInstrColumnName(metadata.getColumnName(cl));
							failedColumns.setInstrValues(values);
							failedInstrColMap.put(columnName, values);
							//failedList.add(failedColumns);
						}
					}

				}
			}
		} catch (SQLException e) {	//Nd-721
			logger.log(Level.SEVERE, "Instructor output fails because of syntax error :" + e.getMessage(), e);
			//failedList.add(failedInstrColMap);
		}
		failedInstrDataMap.put(dataSetId, failedInstrColMap);
		// if(!isDefaultDS){
		failedDs.setInstrQueryOutput(failedInstrDataMap);
		// }else if(isDefaultDS){
		// failedDs.setInstrQueryDefaultDSOutput(failedInstrDataMap);
		// }
		return failedDs;
	}

	/**
	 * This method is called when the instructor evaluates the student queries
	 * after submission. Triggered from instructor during evaluation.
	 * 
	 * @param assignment_id
	 * @param question_id
	 * @throws SQLException
	 * @throws StandardException
	 */
	void evaluateQuestion(int assignment_id, int question_id, String course_Id) {	//Nd-728

		String StudQueryString = "select rollnum,querystring,late_submission_flag from "
				+ "(select * from xdata_student_queries where assignment_id=? and question_id=? AND course_id=?) as C "
				+ "inner join xdata_users on C.rollnum= xdata_users.internal_user_id "
				+ "inner join xdata_roles xdr on xdata_users.internal_user_id = xdr.internal_user_id "
				+ "and xdr.role='student'";

		String resultToUpdate = "&&";
		String oldQueryId = "A" + assignment_id + "Q" + question_id + "S" + 1;// Query
																				// Id
																				// by
																				// student
																				// will
																				// always
																				// be
																				// 1
		TestAnswer test = new TestAnswer();
		try (Connection testCon = (new DatabaseConnection().getTesterConnection(assignment_id)).getTesterConn()) {
			// dbcon = MyConnection.getExistingDatabaseConnection();
			try (Connection dbcon = MyConnection.getDatabaseConnection()) {
				if (dbcon != null) {	//Nd-737
					logger.log(Level.INFO, "Connected successfullly");
				}
				String updateString = "update xdata_student_queries set tajudgement=?, result = ?  where assignment_id=? and question_id = ? and rollnum=? and course_id=?";
				try (PreparedStatement upstmt = dbcon.prepareStatement(updateString)) {
					try (PreparedStatement studQueriesStmt = dbcon.prepareStatement(StudQueryString)) {
						studQueriesStmt.setInt(1, assignment_id);
						studQueriesStmt.setInt(2, question_id);
						studQueriesStmt.setString(3, course_Id);

						logger.log(Level.INFO, "A" + assignment_id + "Q" + question_id + "S" + 1);
						// studQueriesStmt.setInt(2, assignment_id);
						try (ResultSet StudQueries = studQueriesStmt.executeQuery()) {

							HashSet<String> hs = new HashSet<String>();
							GenerateCVC1 cvc = new GenerateCVC1();											
							preProcessForDataGeneration preProcess = new preProcessForDataGeneration();
							
							cvc.setAssignmentId(assignment_id);
							cvc.setQuestionId(question_id);
							cvc.setQueryId(1);
							cvc.setCourseId(course_Id);
							preProcess.initializeConnectionDetails(cvc);
							
							TableMap tm = cvc.getTableMap();
							cvc.closeConn();

							PopulateTestDataGrading p = new PopulateTestDataGrading();
							p.deleteAllTempTablesFromTestUser(testCon);
							p.createTempTables(testCon, assignment_id, question_id);
							// HashMap< String, String> hm=new HashMap<String,
							// String>();
							while (StudQueries.next()) {
								Gson gson = new Gson();
								String json = "";
								int flag = 0;
								hs.clear();
								// SQLParser sqlParser = new SQLParser();
								CCJSqlParserManager pm = new CCJSqlParserManager();

								StatementNode s = null;
								String qry = StudQueries.getString("querystring");
								boolean isLateSubmission = StudQueries.getBoolean("late_submission_flag");
								if (qry != null && !qry.isEmpty()) {	//Nd-774
									String OriginalQry = qry.replaceAll("''", "'");
									logger.log(Level.INFO, "queryString" + OriginalQry);
									qry = OriginalQry.trim().replaceAll("\n+", " ");
									qry = qry.trim().replaceAll(" +", " ");
									qry = qry.replace("NATURAL LEFT OUTER", "NATURAL");
									qry = qry.replace("NATURAL RIGHT OUTER", "NATURAL");
									logger.log(Level.INFO, "Cleansed Query is " + qry);
									
					/*Segment784-795: begins*/				
									try {
																			logger.log(Level.INFO, "queryString" + qry);
										qry = qry.trim().replaceAll("\n+", " ");
										qry = qry.trim().replaceAll(" +", " ");
										// Statement stmt = pm.parse(new
										// StringReader(qry));
									} catch (Exception e) {
										logger.log(Level.SEVERE, e.getMessage(), e);
										// e.printStackTrace();
										// This is set to true to say the
										// question is evaluated
										upstmt.setBoolean(1, true);
										flag = 1;
									}
				/*segment784-795: ends*/					
									if (flag == 0) {
					/*Segment797-859: begins*/				
										String ans = "";
										try {

											FailedDataSetValues failedDs = test.testAnswer(assignment_id, question_id,
													course_Id, OriginalQry, StudQueries.getString("rollnum"),
													"4/" + course_Id + "/" + oldQueryId, isLateSubmission,null);
											logger.log(Level.INFO, "Answer is :" + ans);

											if (failedDs.getStatus().equalsIgnoreCase("Failed")) {
												// This is set to true to say the question is evaluated
												upstmt.setBoolean(1, true);
												// Get DS0
												hs.add(failedDs.getDataSetIdList().get(0));
						/*segment809-849: begins*/						
												Map<String, Map<String, ArrayList<String>>> failedStudDataMap = new HashMap<String, Map<String, ArrayList<String>>>();
												Map<String, Map<String, ArrayList<String>>> failedInstrDataMap = new HashMap<String, Map<String, ArrayList<String>>>();

												String add_datasets = "insert into xdata_detectdataset values (?,?,?,?,?,?,?)";

												try (PreparedStatement add = dbcon.prepareStatement(add_datasets)) {	//Nd-812
													for (int i = 0; i < failedDs.getDataSetIdList().size(); i++) {
														ArrayList<FailedColumnValues> failedList = new ArrayList<FailedColumnValues>();
														Map<String, ArrayList<String>> failedColMap = new HashMap<String, ArrayList<String>>();

														logger.log(Level.INFO, "Inside operator");
														String dataset = failedDs.getDataSetIdList().get(i);
														if (dataset.startsWith("DS")) {	//Nd-824
															if (dataset.equalsIgnoreCase("DS_Default")) {
																p.createTempTableData(dbcon, testCon, assignment_id,
																		question_id, course_Id);

															} else {
																p.fetchAndPopulateTestDatabase(dbcon, testCon,
																		assignment_id, question_id, 1, course_Id,
																		dataset, tm);

															}

														} else {
															p.createTempTableWithDefaultData(dbcon, testCon,
																	assignment_id, question_id, course_Id, dataset);
														}
														failedDs = this.getStudentOutput(testCon, dataset, OriginalQry,
																failedList, failedDs, failedStudDataMap, false);
														failedDs = this.getInstructorOutput(testCon, dataset,
																failedDs.getInstrQuery(), failedList, failedDs,
																failedInstrDataMap, false);
														logger.log(Level.INFO, "\n connection val: " + dbcon);
														failedDs.getDsValueMap().put(dataset, failedList);
														json = gson.toJson(failedDs);
														logger.log(Level.INFO, "JSON VALUE OF DataSetValue field :"
																+ json + "\n $$$$$$$$$$$  connection :7 : " + dbcon);

														add.setString(1, StudQueries.getString("rollnum"));
														add.setString(2, oldQueryId);
														add.setString(3, dataset);
														add.setString(4, json);
														add.setInt(5, assignment_id);
														add.setInt(6, question_id);
														add.setString(7, course_Id);
														add.executeUpdate();
														// loop ends
													} // for each DSloop ends
												} // try block for statement
					/*segment809-849: ends*/							
													// ends
											} else {	//Nd-850
												upstmt.setBoolean(1, true);
											}
											json = gson.toJson(failedDs);
											logger.log(Level.INFO, "JSON VALUE OF DataSetValue field :" + json
													+ "\n connection : " + dbcon);

											upstmt.setString(2, json);

										} catch (Exception e) {
											logger.log(Level.SEVERE, "Exception caught here: " + e.getMessage(), e);
											// e.printStackTrace();
											upstmt.setBoolean(1, true);
										}
				/*segment797-859: ends*/						
									}
									upstmt.setString(2, json);
									// upstmt.setString(3,course_Id);
									if (flag == 1) {	//Nd-861
										DatabaseHelper.InsertIntoScores(dbcon, assignment_id, question_id, 1, course_Id,
												100, StudQueries.getString("rollnum"), "", 0F);
									}
									upstmt.setInt(3, assignment_id);
									upstmt.setInt(4, question_id);
									// upstmt.setInt(3, question_id);
									upstmt.setString(5, StudQueries.getString(1));
									upstmt.setString(6, course_Id);
									upstmt.executeUpdate();
								} else {	//Nd-870
									DatabaseHelper.InsertIntoScores(dbcon, assignment_id, question_id, 1, course_Id,
											100, StudQueries.getString("rollnum"), "", 0F);
									upstmt.setBoolean(1, false);
									upstmt.setString(2, null);
									upstmt.setInt(3, assignment_id);
									upstmt.setInt(4, question_id);
									upstmt.setString(5, StudQueries.getString(1));
									upstmt.setString(6, course_Id);

									upstmt.executeUpdate();
								}
							}

							p.deleteAllTempTablesFromTestUser(testCon);
						} // try block for update resultset ends
					} // try block for update stment ends
					catch (Exception e) {
						logger.log(Level.SEVERE,
								"Exception caught at the end of TestAssignment method: " + e.getMessage(), e);
						// e.printStackTrace();
						// throw e;
					}
				} // try block for studQueries stmt ends
			} // try block for testcon ends
			catch (Exception e) {
				logger.log(Level.SEVERE, "Exception caught at TestAssginment.evaluate method : " + e.getMessage(), e);
				// e.printStackTrace();
				// throw e;
			}
		} // try block for dbcon ends
		catch (Exception e) {
			logger.log(Level.SEVERE, "Exception caught at end of TestAssignment.evaluate method: " + e.getMessage(), e);
			// e.printStackTrace();
			// throw e;
		}
	}

	/**
	 * This method is called when the student submits the query in
	 * non-interactive mode. The student query is tested correct or wrong for
	 * DS0 alone and status is returned.
	 * 
	 * @param args
	 * @return
	 * @throws Exception
	 */
	public QueryStatusData testQuery(String[] args) throws SQLException {//Nd-894
		// test here
		QueryStatusData queryStatus = new QueryStatusData();
		queryStatus.Status = QueryStatus.Correct;
		boolean isQueryExists = false;
		PopulateTestDataGrading populateTestData = new PopulateTestDataGrading();
		int assignment_id = Integer.parseInt(args[0]);
		int question_id = Integer.parseInt(args[1]);
		String rollnum = args[2];
		String courseId = args[3];

		try (Connection dbcon = MyConnection.getDatabaseConnection()) {
			try (Connection testCon = (new DatabaseConnection().getTesterConnection(assignment_id)).getTesterConn()) {

				if (dbcon != null) {
					logger.log(Level.INFO, "Connected successfullly");
				}

				QueryStatus status = QueryStatus.Incorrect;
				// SQLParser sqlParser = new SQLParser();
				CCJSqlParserManager pm = new CCJSqlParserManager();
				String StudQueryString = "select querystring from xdata_student_queries where assignment_id=? and question_id=? and rollnum=? and course_id=?";

				TestAnswer test = new TestAnswer();
				try (PreparedStatement studQueryStmt = dbcon.prepareStatement(StudQueryString)) {
					studQueryStmt.setInt(1, assignment_id);
					studQueryStmt.setInt(2, question_id);
					studQueryStmt.setString(3, rollnum);
					studQueryStmt.setString(4, courseId);
					logger.log(Level.INFO, "A" + assignment_id + "Q" + question_id + "S" + 1);

					try (ResultSet studentQuery = studQueryStmt.executeQuery()) {
						if (studentQuery.next()) {
							String qry = studentQuery.getString("querystring");
							String testQuery = qry;
							int flag = 0;
							// SQLParser sqlParser = new SQLParser();
							StatementNode s = null;
						
			/*segment928-959: begins*/			
							if (qry != null && !qry.isEmpty() && testQuery != null && !testQuery.isEmpty()) {
								isQueryExists = true;
								String OriginalQry = qry.replaceAll("''", "'");
								logger.log(Level.INFO, "queryString" + OriginalQry);
								qry = OriginalQry.trim().replaceAll("\n+", " ");
								qry = qry.trim().replaceAll(" +", " ");
								qry = qry.replace("NATURAL LEFT OUTER", "NATURAL");
								qry = qry.replace("NATURAL RIGHT OUTER", "NATURAL");

								logger.log(Level.INFO, "Cleansed Query is " + qry);
			/*segment941-958: begins*/					
								try {
									logger.log(Level.INFO, "queryString" + testQuery);
									testQuery = testQuery.trim().replaceAll("\n+", " ");
									testQuery = testQuery.trim().replaceAll(" +", " ");

									populateTestData.deleteAllTempTablesFromTestUser(testCon);
									populateTestData.createTempTables(testCon, assignment_id, question_id);
									try (PreparedStatement testStatement = testCon.prepareStatement(testQuery)) {
										testStatement.executeQuery();
									}
								} catch (Exception e) {
									logger.log(Level.SEVERE, e.getMessage(), e);
									// e.printStackTrace();
									queryStatus.ErrorMessage = e.getMessage();
									queryStatus.Status = QueryStatus.Error;
									flag = 1;
								}
		/*segment941-958: ends*/						
			/*Segment959-966: begins*/
								if (flag == 0) {
									try {
										String queryId = "A" + assignment_id + "Q" + question_id + "S" + 1;
										status = test.testQueryAnswer(assignment_id, question_id, courseId, OriginalQry,
												rollnum, "4/" + courseId + "/" + queryId);
										// status = QueryStatus.Correct;
									} catch (Exception e) {
										logger.log(Level.SEVERE, "Exception caught here:" + e.getStackTrace(), e);
										// e.printStackTrace();
									}
								}
			/*Segment959-966: ends*/					
							}
			/*segment928-959: ends*/				
						}
					} // try block for resultset studentQuery ends
				} // try block for studQueryStmt ends
				queryStatus.Status = status;

				if (!isQueryExists) {//Nd-968
					String qryUpdate = "update xdata_student_queries set verifiedcorrect = false where assignment_id = '"
							+ assignment_id + "' and question_id='" + question_id + "' and rollnum = '" + rollnum
							+ "' and course_id='" + courseId + "'";
					try (PreparedStatement pstmt3 = dbcon.prepareStatement(qryUpdate)) {
						pstmt3.executeUpdate();
					}
					queryStatus.Status = QueryStatus.NoDataset;
				} else if (queryStatus.Status == QueryStatus.Correct) {//Nd-974
					String qryUpdate = "update xdata_student_queries set verifiedcorrect = true where assignment_id = '"
							+ assignment_id + "' and question_id='" + question_id + "' and rollnum = '" + rollnum
							+ "' and course_id='" + courseId + "'";
					try (PreparedStatement pstmt3 = dbcon.prepareStatement(qryUpdate)) {
						pstmt3.executeUpdate();
					}
				} else {//Nd-980
					String qryUpdate = "update xdata_student_queries set verifiedcorrect = false where assignment_id = '"
							+ assignment_id + "' and question_id='" + question_id + "' and rollnum = '" + rollnum
							+ "' and course_id='" + courseId + "'";
					try (PreparedStatement pstmt3 = dbcon.prepareStatement(qryUpdate)) {
						pstmt3.executeUpdate();
					}
				}
				try {
					populateTestData.deleteAllTempTablesFromTestUser(testCon);
				} catch (Exception e) {
					logger.log(Level.SEVERE, e.getMessage(), e);
				}

				testCon.close();
				dbcon.close();
			} // try block for testconn ends
		} // try block for conn ends
			// logger.log(Level.INFO,"TestAssignment - DS0 MODE :
		return queryStatus;
	}

	/**
	 * This method is called when instructor evaluates the assignment.
	 * Instructor grades the student queries as correct or incorrect based on
	 * answer
	 * 
	 * @param args
	 */
	public static void entry(String[] args) {//Nd-994
		TestAssignment ta = new TestAssignment();
		try {
			int assignment_id = Integer.parseInt(args[0]);
			int question_id = Integer.parseInt(args[1]);
			String course_id = args[2];
			
			try (Connection dbcon = MyConnection.getDatabaseConnection()) {
				ta.newEvaluateAssignment(dbcon, args);
			} 
			//ta.evaluateQuestion(assignment_id, question_id, course_id);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			logger.log(Level.SEVERE, e.getMessage(), e);
			// e.printStackTrace();
		}
	}

	/**
	 * This method is called when the query equivalence is tested in TESTER mode
	 * 
	 * @param args
	 */
	public String checkQueryEquivalence(int assignment_id, int question_id, String course_Id) throws Exception {//Nd-1007
		String status = "pass";
		TestAnswer test = new TestAnswer();
		boolean equivalenceStatus = true;
		try (Connection testCon = (new DatabaseConnection().getTesterConnection(assignment_id)).getTesterConn()) {

			try (Connection dbcon = MyConnection.getDatabaseConnection()) {
				if (dbcon != null) {
					logger.log(Level.INFO, "Connected successfullly");
				}
				logger.log(Level.INFO, "A" + assignment_id + "Q" + question_id + "S" + 1);
				GenerateCVC1 cvc = new GenerateCVC1();											
				preProcessForDataGeneration preProcess = new preProcessForDataGeneration();
				
				cvc.setAssignmentId(assignment_id);
				cvc.setQuestionId(question_id);
				cvc.setQueryId(1);
				cvc.setCourseId(course_Id);
				preProcess.initializeConnectionDetails(cvc);
				
				TableMap tm = cvc.getTableMap();
				cvc.closeConn();

				PopulateTestDataGrading p = new PopulateTestDataGrading();
				p.deleteAllTempTablesFromTestUser(testCon);
				p.createTempTables(testCon, assignment_id, question_id);
				Gson gson = new Gson();
				ArrayList<String> mutants = new ArrayList<String>();
				int index = 0;

				String json = "";
				String ans = "";
				String add_failed_datasets = "update xdata_qinfo set equivalence_failed_datasets = ?, equivalenceStatus = ? where assignment_id = ? and question_id=? and course_id=?";
				String mutantQuery = "select sql from xdata_instructor_query where assignment_id=? and question_id=? and course_id=?";
				try (PreparedStatement sqlQuery = dbcon.prepareStatement(mutantQuery)) {
					sqlQuery.setInt(1, assignment_id);
					sqlQuery.setInt(2, question_id);
					sqlQuery.setString(3, course_Id);
					ResultSet rSet = sqlQuery.executeQuery();
					while (rSet.next()) {
						if (index != 0) {
							mutants.add(rSet.getString("sql"));
						}
						index++;
					}
				}
				try (PreparedStatement update = dbcon.prepareStatement(add_failed_datasets)) {
					try {
						ArrayList<FailedDataSetValues> failedDSToJson = new ArrayList<FailedDataSetValues>();

						Map<Integer, FailedDataSetValues> failedDsMap = test.checkQueryEquivalence(assignment_id,
								question_id, course_Id);
						// for(FailedDataSetValues failedDs:listOfFailedDS){
						Iterator it = failedDsMap.keySet().iterator();
						while (it.hasNext()) {
							FailedDataSetValues failedDs = failedDsMap.get(it.next());
							logger.log(Level.INFO, "Answer is :" + ans);
			/*Segment1061-1101: begins*/				
							if (failedDs.getStatus().equalsIgnoreCase("Failed")) {
								// newFailedDSValue.setQuery_id(failedDs.getQuery_id());
								status = "failed";
								equivalenceStatus = false;
								// This is set to true to say the question is
								// evaluated
								// upstmt.setBoolean(1, true);
								// Get DS0
								Map<String, Map<String, ArrayList<String>>> failedStudDataMap = new HashMap<String, Map<String, ArrayList<String>>>();
								Map<String, Map<String, ArrayList<String>>> failedInstrDataMap = new HashMap<String, Map<String, ArrayList<String>>>();

								for (int i = 0; i < failedDs.getDataSetIdList().size(); i++) {

									FailedDataSetValues newFailedDSValue = new FailedDataSetValues();
									newFailedDSValue.setStatus(status);
									newFailedDSValue.setQuery_id(failedDs.getQuery_id());
									ArrayList<FailedColumnValues> failedList = new ArrayList<FailedColumnValues>();
									Map<String, ArrayList<String>> failedColMap = new HashMap<String, ArrayList<String>>();

									logger.log(Level.INFO, "Inside operator");
									String dataset = failedDs.getDataSetIdList().get(i);
									newFailedDSValue.setDataSetId(dataset);

									if (dataset.startsWith("DS")) {
										if (dataset.equalsIgnoreCase("DS_Default")) {
											p.createTempTableData(dbcon, testCon, assignment_id, question_id,
													course_Id);
										} else {
											p.fetchAndPopulateTestDatabase(dbcon, testCon, assignment_id, question_id,
													failedDs.getQuery_id(), course_Id, dataset, tm);
										}

									} else {
										p.createTempTableWithDefaultData(dbcon, testCon, assignment_id, question_id,
												course_Id, dataset);
									}
									newFailedDSValue = this.getStudentOutput(testCon, dataset, mutants.get(0),
											failedList, newFailedDSValue, failedStudDataMap, false);
									newFailedDSValue = this.getInstructorOutput(testCon, dataset,
											failedDs.getInstrQuery(), failedList, newFailedDSValue, failedInstrDataMap,
											false);

									newFailedDSValue.getDsValueMap().put(dataset, failedList);
									failedDSToJson.add(newFailedDSValue);
									
								} // for each DSloop ends
							} // if status failed loop ends
							else {//Nd-1100
								// upstmt.setBoolean(1, true);
							}
		/*Segment1061-1101: ends*/					
							logger.log(Level.INFO, "JSON VALUE OF DataSetValue field :" + json);

						} // while iterator loop ends
						json = gson.toJson(failedDSToJson);
						update.setString(1, json);
						update.setBoolean(2, equivalenceStatus);
						update.setInt(3, assignment_id);
						update.setInt(4, question_id);
						update.setString(5, course_Id);
						update.execute();
					} catch (Exception e) {//Nd-1110
						logger.log(Level.SEVERE, "Exception caught here: " + e.getMessage(), e);
						e.printStackTrace();
					}

				}
				p.deleteAllTempTablesFromTestUser(testCon);
			} // try block for testcon ends
		} // try block for dbcon ends
		return status;
	}

	/*
	 * method for TESTING - parsing in batch. Assumption: queries are stored in
	 * file <srcFileName>, the non-parsing queries are stored at the end of
	 * execution in file <tarFileName>
	 * 
	 * To be deleted
	 */

	public static void readQueriesFromFileParseAndTest() {
		String srcFileName = "/home/shree/Documents/Assignment1/Q2/Records10.txt",
				tarFileName = "/home/shree/Documents/Assignment1/Q2/BadQueries.txt";

		TestAnswer ta = new TestAnswer();
		try {
			BufferedReader reader = new BufferedReader(new FileReader(srcFileName));
			PrintWriter writer = new PrintWriter(tarFileName);
			String line = null;
			String query = "";

			while ((line = reader.readLine()) != null) {
				if (line.isEmpty()) {
					if (query.trim() != "") {
						String serialNum = query.substring(0, query.indexOf(")") + 1);
						String actualQuery = query.substring(query.indexOf(")") + 1, query.indexOf(";"));
						Vector<String> columnmismatch = new Vector();
						System.out.println(serialNum);
						try {
							// ta.testQuery(args)
							Connection dbcon = getDBConnectionToTest();
							Connection testCon = getTesterConnectionToTest();

							HashMap<String, String> mutants = new HashMap<String, String>();
							mutants.put("q2", actualQuery);
							Vector<String> result = TestAnswer.checkAgainstOriginalQuery(mutants, "DS_TEST",
									"select student.id, name from student, takes, course where student.id=takes.id and takes.course_id = course.course_id and course.dept_name=\'Comp. Sci.\' and year<2010"
											+ " intersect select student.id, name from student, takes, course where student.id=takes.id and takes.course_id = course.course_id and course.dept_name=\'Comp. Sci.\' and year>2010;"
											+ " ",
									"", true, columnmismatch, testCon);

							if (result != null && !result.isEmpty()) {
								System.out.println("Bad query " + serialNum + " : " + actualQuery);
								writer.println(serialNum + actualQuery);
							} else {
								System.out.println("good query " + serialNum + " : " + actualQuery);
							}
						} catch (Exception e) {
							System.out.println("Bad query " + serialNum + " : " + actualQuery);
							writer.println(serialNum + actualQuery);
						}
					}
					query = "";
				} else {
					query += line;
					// System.out.println(query);
				}

			}
			reader.close();
			writer.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		TestAssignment ta = new TestAssignment();
		try {
			readQueriesFromFileParseAndTest();
		} catch (Exception e) {
			logger.log(Level.SEVERE, e.getMessage(), e);
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * This method is used for TESTING threads handled by XDATA during student
	 * submission. - LOAD TESTING
	 * 
	 * @param args
	 * @return
	 * @throws Exception
	 */
	public QueryStatusData testThreadsForQuery(String[] args) throws Exception {
		// test here
		QueryStatusData queryStatus = new QueryStatusData();
		queryStatus.Status = QueryStatus.Correct;
		boolean isQueryExists = false;
		PopulateTestDataGrading populateTestData = new PopulateTestDataGrading();
		int assignment_id = Integer.parseInt(args[0]);
		int question_id = Integer.parseInt(args[1]);
		String rollnum = args[2];
		String courseId = args[3];
		try (Connection dbcon = getDBConnectionToTest()) {
			try (Connection testCon = getTesterConnectionToTest()) {

				if (dbcon != null) {
					logger.log(Level.INFO, "Connected successfullly");
				}

				QueryStatus status = QueryStatus.Incorrect;
				// SQLParser sqlParser = new SQLParser();
				CCJSqlParserManager pm = new CCJSqlParserManager();
				String StudQueryString = "select querystring from xdata_student_queries where assignment_id=? and question_id=? and rollnum=? and course_id=?";

				TestAnswer test = new TestAnswer();
				try (PreparedStatement studQueryStmt = dbcon.prepareStatement(StudQueryString)) {
					studQueryStmt.setInt(1, assignment_id);
					studQueryStmt.setInt(2, question_id);
					studQueryStmt.setString(3, rollnum);
					studQueryStmt.setString(4, courseId);
					logger.log(Level.INFO, "A" + assignment_id + "Q" + question_id + "S" + 1);

					try (ResultSet studentQuery = studQueryStmt.executeQuery()) {
						if (studentQuery.next()) {
							String qry = studentQuery.getString("querystring");
							String testQuery = qry;
							int flag = 0;
							// SQLParser sqlParser = new SQLParser();
							StatementNode s = null;
			/*segment1207-1238: begigns*/				
							if (qry != null && !qry.isEmpty() && testQuery != null && !testQuery.isEmpty()) {
								isQueryExists = true;
								String OriginalQry = qry.replaceAll("''", "'");
								logger.log(Level.INFO, "queryString" + OriginalQry);
								qry = OriginalQry.trim().replaceAll("\n+", " ");
								qry = qry.trim().replaceAll(" +", " ");
								qry = qry.replace("NATURAL LEFT OUTER", "NATURAL");
								qry = qry.replace("NATURAL RIGHT OUTER", "NATURAL");
								/*
								 * if(qry.toLowerCase().contains("year")){
								 * qry=qry.replaceAll("year","year1");
								 * qry=qry.replaceAll("Year","year1");
								 * qry=qry.replaceAll("YEAR","year1"); }
								 */
								logger.log(Level.INFO, "Cleansed Query is " + qry);
			/*segment1220-1237: begins*/
								try {
									logger.log(Level.INFO, "queryString" + testQuery);
									testQuery = testQuery.trim().replaceAll("\n+", " ");
									testQuery = testQuery.trim().replaceAll(" +", " ");
									
									populateTestData.deleteAllTempTablesFromTestUser(testCon);
									populateTestData.createTempTablesForTestThread(testCon, assignment_id, question_id);
									try (PreparedStatement testStatement = testCon.prepareStatement(testQuery)) {
										testStatement.executeQuery();
									}
								} catch (Exception e) {
									logger.log(Level.SEVERE, e.getMessage(), e);
									// e.printStackTrace();
									queryStatus.ErrorMessage = e.getMessage();
									queryStatus.Status = QueryStatus.Error;
									flag = 1;
								}
		/*segment1220-1237:ends*/
		/*segment1238-1245 :begins*/
								if (flag == 0) {
									try {
										String queryId = "A" + assignment_id + "Q" + question_id + "S" + 1;
										status = test.testQueryAnswerForTestThreads(assignment_id, question_id,
												courseId, OriginalQry, rollnum, "4/" + courseId + "/" + queryId);
										// status = QueryStatus.Correct;
									} catch (Exception e) {
										logger.log(Level.SEVERE, "Exception caught here:" + e.getStackTrace(), e);
										// e.printStackTrace();
									}
								}
		/*segment1238-1245 : ends*/						
							}
		/*segment1207-1238 : ends*/						
						}

					} // try block for resultset studentQuery ends
				} // try block for studQueryStmt ends
				queryStatus.Status = status;

				if (!isQueryExists) {
					String qryUpdate = "update xdata_student_queries set verifiedcorrect = false where assignment_id = '"
							+ assignment_id + "' and question_id='" + question_id + "' and rollnum = '" + rollnum
							+ "' and course_id='" + courseId + "'";
					try (PreparedStatement pstmt3 = dbcon.prepareStatement(qryUpdate)) {
						pstmt3.executeUpdate();
					}
					queryStatus.Status = QueryStatus.NoDataset;
				} else if (queryStatus.Status == QueryStatus.Correct) {
					String qryUpdate = "update xdata_student_queries set verifiedcorrect = true where assignment_id = '"
							+ assignment_id + "' and question_id='" + question_id + "' and rollnum = '" + rollnum
							+ "' and course_id='" + courseId + "'";
					try (PreparedStatement pstmt3 = dbcon.prepareStatement(qryUpdate)) {
						pstmt3.executeUpdate();
					}
				} else {
					String qryUpdate = "update xdata_student_queries set verifiedcorrect = false where assignment_id = '"
							+ assignment_id + "' and question_id='" + question_id + "' and rollnum = '" + rollnum
							+ "' and course_id='" + courseId + "'";
					try (PreparedStatement pstmt3 = dbcon.prepareStatement(qryUpdate)) {
						pstmt3.executeUpdate();
					}
				}

				populateTestData.deleteAllTempTablesFromTestUser(testCon);

				testCon.close();
			} // try block for testconn ends
			dbcon.close();

		} // try block for conn ends
			// logger.log(Level.INFO,"TestAssignment - DS0 MODE :
			// "+ro.kifs.diagnostic.Connection.getStillOpenedConnsStackTraces());

		return queryStatus;
	}

	/**
	 * This method is for TESTING STUDENT CONNECTIONS / SUBMISSIONS - LOAD
	 * TESTING
	 * 
	 * @return
	 * @throws SQLException
	 */

	public static Connection getDBConnectionToTest() throws SQLException {

		Connection conn = null;
		try {
			System.out.println("Number of connections = " + Thread.currentThread());

			Class.forName("org.postgresql.Driver");
			conn = DriverManager.getConnection("jdbc:postgresql://localhost:5432/xdata", "testing1", "password");
		} catch (ClassNotFoundException ex) {
			System.out.println("Error: unable to load driver class!");
			System.exit(1);
		}
		return conn;

	}

	/**
	 * This method helpd in getting connections for TESTING STUDENT CONNECTIONS
	 * / SUBMISSIONS - LOAD TESTING
	 * 
	 * @return
	 * @throws SQLException
	 */
	public static Connection getTesterConnectionToTest() throws SQLException {

		Connection conn = null;
		try {
			Class.forName("org.postgresql.Driver");
			conn = DriverManager.getConnection("jdbc:postgresql://localhost:5432/xdata", "testing2", "password");
		} catch (ClassNotFoundException ex) {
			System.out.println("Error: unable to load driver class!");
			System.exit(1);
		}
		return conn;

	}
}
